import processing.opengl.*;
import javax.media.opengl.*;
import peasy.*;
import controlP5.*;

ControlP5 controlP5;
PMatrix3D currCameraMatrix;
PGraphics3D g3; 
ControlGroup l;
Slider mySlider2; 

Textlabel mySmallTextlabel;
Integrator slide; 

PeasyCam cam;
PGraphicsOpenGL pgl;
GL gl;

int nodeCount; 
Node[] nodes = new Node[100]; 
Node selection;
int edgeCount; 
Edge[] edges = new Edge[500];
HashMap nodeTable = new HashMap(); 

//Sketch Colors 
public int myColorRect = 200;

float cLength = 70; 
//static final color nodeColor = #F0C070; 
static final color nodeColor = #05B2FF; 
static final color fontColor = #aaaaaa; 
static final color selectColor = #FFAF2E; 
static final color fixedColor = #05DCFF; 
static final color edgeColor = #FFFFFF; 
color bgColor = color(0, 0, 0, 0);

// visual effects
float[] rotations = new float[3]; 

//Global Parameters: 
String node1; 
String node2; 
int maxCount = -1; 
int popular = 0;
int lastpopular = -1;
PFont font; 

PGraphics buffer;   // buffer for 3D picking...important!!!!

void setup() {
  size(1280, 800, OPENGL); 
  hint(DISABLE_OPENGL_2X_SMOOTH); 
  hint(ENABLE_OPENGL_4X_SMOOTH); 
  hint(ENABLE_DEPTH_SORT); 

  font = loadFont("HelveticaNeue-58.vlw"); 

  //LoadData function: 
  loadData();

  //Camera Function: 
  cam = new PeasyCam(this, 300);
  cam.setMinimumDistance(50);
  cam.setMaximumDistance(6000);

  buffer = createGraphics(width, height, P3D);
  slide = new Integrator(780, 0.3, 0.8);

  setupGUI();
}
void draw() 
{    
  slide.update();
  rotations = cam.getRotations(); // get rotations
  
  background(bgColor); 
  
  //this moves the interface up and down
  if (mouseY > 680)
  {
    slide.target(680);
  }
  else
  {
    slide.target(780);
  }
  if (l.position().y != slide.value)
  {
    l.position().y = slide.value;
  }


  // openGL drawing
  pgl = (PGraphicsOpenGL) g;
  gl = pgl.gl;
  pgl.beginGL();
  
  // parameters
  gl.glDepthMask(false); 
  gl.glEnable(GL.GL_DEPTH_TEST);
  gl.glClear(GL.GL_DEPTH_BUFFER_BIT);  
  gl.glEnable ( GL.GL_LINE_SMOOTH );

  gl.glEnable(GL.GL_BLEND);
  gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE);

  gl.glEnable (GL.GL_LINE_SMOOTH);
  gl.glHint (GL.GL_LINE_SMOOTH_HINT, GL.GL_NICEST);
  
  
  // update edges and nodes
  for (int i = 0; i < edgeCount; i++)
  {
    edges[i].relax();
  }
  for (int i = 0; i < nodeCount; i++)
  {
    nodes[i].relax();
  }
  
  // check all nodes, update them, check if a node count has gone up, if so, make it the new maxCount
  // then set popular to that node
  for (int i = 0; i < nodeCount; i++)
  {
    nodes[i].update(); 
    if (nodes[i].count > maxCount)
    {
      maxCount = nodes[i].count; 
      popular = i;
    }
  }
  
  // draw edges and nodes
  for (int i = 0; i < edgeCount; i++)
  {
    edges[i].draw();
  }
  for (int i = 0; i < nodeCount; i++)
  {
    nodes[i].draw(this.g);
  } 
  
  // check who is most popular
  if (popular != lastpopular)
  {
    nodes[popular].fixed = true; // if popular is not the same as lastpopular, set node at index popular to fixed
    nodes[popular].loc.x = 0; // make it the center of the universe to rotate around it
    nodes[popular].loc.y = 0; 
    nodes[popular].loc.z = 0; 
    if (lastpopular >=0)
    {
      nodes[lastpopular].fixed = false; // set previously popular node to not fixed anymore
    }
    lastpopular = popular; // save the newly popular node index in lastpopular
  } 

//  stroke(200, 50); 
//  strokeWeight(2); 
//  line(-100, 0, 0, 100, 0, 0); 
//  line(0, -100, 0, 0, 100, 0); 
//  line(0, 0, -100, 0, 0, 100); 

  pgl.endGL(); // end OpenGL
  
  gui(); // allows gui to be drawn with peasy cam
}


void loadData()
{
  String[] lines = loadStrings("lorizzle.txt");

  //make the text into a single String Object 
  String sLine = join(lines, " "); 
  sLine = sLine.replaceAll("--", "\u2014");

  //Split into phases using any of the provided tokens
  String[] phrases = splitTokens(sLine, ".,;:?!\u2014\"");
  
  // take array of phrases and split each into it's words
  for (int i = 0; i < phrases.length; i++)
  {
    println(phrases[i]);
    
    //make this phrase lower case
    String phrase = phrases[i].toLowerCase(); 
    
    //Split each phrase into individual words at one or more spaces
    String[] words = splitTokens(phrase, " "); 
    for (int w = 0; w < words.length-1; w++)
    {
      addEdge(words[w], words[w+1]); // add edge for word between from and to
    }
  }
  //an example of how to add nodes with a connection
  //  addEdge("visual", "map");
}

void addEdge(String fromLabel, String toLabel)
{
  // make two new nodes and increment their counters (this counts the total occourance of the Node with name)
  Node from = findNode(fromLabel); 
  Node to = findNode(toLabel); 
  from.increment(); 
  to.increment(); 

  //Check to see whether this Edge already exists, if so, increment count for edge i
  for (int i = 0; i < edgeCount; i++)
  {
    if (edges[i].from == from && edges[i].to == to)
    {
      edges[i].increment();
      return; // done
    }
  }

  // If it doesn't exist yet, make new edge
  Edge e = new Edge(from, to); 
  e.increment(); 
  if (edgeCount == edges.length)
  {
    edges = (Edge[]) expand(edges);
  }
  edges[edgeCount++] = e; //save the new edge
}

// looks to see if a node already exists in nodeTable hashtable
Node findNode(String label)
{
  label = label.toLowerCase(); 
  Node n = (Node) nodeTable.get(label); 
  if (n == null)
  {
    return addNode(label);
  }
  return n;
}

// add a new node to nodes and nodeTable hashtable
Node addNode(String label)
{
  Node n = new Node(label);
  if (nodeCount == nodes.length)
  {
    nodes = (Node[]) expand(nodes);
  }
  nodeTable.put(label, n); 
  nodes[nodeCount++] = n; 
  return n;
}

// picker functions
//
color getColor(int id) {
  return -(id + 2);
}

// color -2 gives 0, etc.
int getId(color c) {
  return -(c + 2);
}

